@@ -1438,8 +1438,9 @@ class Imagen(nn.Module):
         cond_scale = 1.,
         lowres_sample_noise_level = None,
         stop_at_unet_number = None,
+        return_all_unet_outputs = False,
         return_pil_images = False,
-        device = None
+        device = None,
     ):
         device = default(device, lambda: next(self.parameters()).device)
 
@@ -1454,7 +1455,8 @@ class Imagen(nn.Module):
         assert not (not self.condition_on_text and exists(text_embeds)), 'imagen specified not to be conditioned on text, yet it is presented'
         assert not (exists(text_embeds) and text_embeds.shape[-1] != self.text_embed_dim), f'invalid text embedding dimension being passed in (should be {self.text_embed_dim})'
 
-        img = None
+        outputs = []
+
         is_cuda = next(self.parameters()).is_cuda
         device = next(self.parameters()).device
 
@@ -1487,14 +1489,22 @@ class Imagen(nn.Module):
                     noise_scheduler = noise_scheduler
                 )
 
+                outputs.append(img)
+
             if exists(stop_at_unet_number) and stop_at_unet_number == unet_number:
                 break
 
-        if not return_pil_images:
-            return img
+        output_index = -1 if not return_all_unet_outputs else slice(None) # either return last unet output or all unet outputs
 
-        pil_images = list(map(T.ToPILImage(), img.unbind(dim = 0)))
-        return pil_images # now you have a bunch of pillow images you can just .save(/where/ever/you/want.png)
+        if not return_pil_images:
+            return outputs[output_index]
+
+        if not return_all_unet_outputs:
+            outputs = outputs[-1:]
+
+        pil_images = list(map(lambda img: list(map(T.ToPILImage(), img.unbind(dim = 0))), outputs))
+
+        return pil_images[output_index] # now you have a bunch of pillow images you can just .save(/where/ever/you/want.png)
 
     def p_losses(self, unet, x_start, times, *, noise_scheduler, lowres_cond_img = None, lowres_aug_times = None, text_embeds = None, text_mask = None, noise = None, times_next = None):
         noise = default(noise, lambda: torch.randn_like(x_start))
